/* Mail transport interface.
 *
 * PegSoft mail library (c) 2006 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/** \file mail/transport.h
  * \brief Mail transport interface header file. */

#ifndef MAIL_TRANSPORT_H
#define MAIL_TRANSPORT_H

/*************************************************************************/

#include <modules/interface.h>

/*************************************************************************/

namespace mail 
{

/*************************************************************************/

class Mail;

/*************************************************************************/

/// Mail transport interface.

/** This interface is used by modules to implement mail transport modules,
  * that can be used by the mail subsystem to deliver mails by different
  * means.
  *
  * This interface must be exposed through the Module::interface() function.
  * The deliver() function needs to be implemented.
  */
  
class Transport : public modules::ErrorInterface
{
	public:
		/// Unique interface identifier
		static modules::PSIID const IID = 0x45724731;
			
		/// Delivers an e-mail.
		/** Delivers an e-mail. How this e-mail is sent is up to the
		  * implementer.
		  *
		  * \param[in] mail The e-mail to send. A copy of this object
		  * must be done if needed as it is not guaranteed to be valid
		  * after the function returns.
		  * \param[out] temp If the function fails, the boolean value
		  * pointed by this parameter must be set to true if the error
		  * is temporary (and thus the e-mail could be delivered later)
		  * and false if the error is permanent.
		  * \return true if the e-mail has been delivered or is queued
		  * for delivery, false otherwise. The error() function of the
		  * interface must return a description of the error that
		  * happened in the latter case. */
		virtual bool deliver(Mail const &mail, bool *temp) = 0;
};

/*************************************************************************/

} /* namespace mail */

/*************************************************************************/

#endif /* MAIL_TRANSPORT_H */
