/* absolute_path().
 *
 * PegSoft miscellaneous library (c) 2004 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include <system/headers.h>

#include "misc/misc.h"
#include "misc/string.h"

using namespace std;

/*************************************************************************/

namespace misc
{

/*************************************************************************/

void absolute_path(char *buf, size_t len, char const *default_dir,
		char const *filename)
{
	/* If the first character is not /, it is not an absolute path */
	if (*filename != '/' && default_dir && *default_dir) {
		char *end = stpncpyzt(buf, default_dir, len);
		if (*(end-1) != '/')
			end = stpncpyzt(end, "/", len-(end-buf));
		strncpyzt(end, filename, len-(end-buf));
	} else {
		strncpyzt(buf, filename, len);
	}
}

/*************************************************************************/

cstring absolute_path(char const *default_dir, char const *filename)
{
	/* If the first character is not /, it is not an absolute path */
	if (*filename != '/' && default_dir && *default_dir) 
	{
		cstring result;
		result.reserve(strlen(default_dir) + strlen(filename) + 1);
		
		char *end = stpcpy(result, default_dir);
		if (*(end-1) != '/')
			*end++ = '/';
		strcpy(end, filename);
		
		return result;
	} else {
		return cstring(filename);
	}
}

/*************************************************************************/

std::string absolute_path(std::string const &default_dir,
		std::string const &filename)
{
	/* If the first character is not /, it is not an absolute path */
	if (!filename.empty() && filename[0] != '/' && !default_dir.empty()) 
	{
		string result;
		result.reserve(default_dir.size() + filename.size() + 1);
		
		result.append(default_dir);
		if (result[result.size() - 1] != '/')
			result.append("/");
		result.append(filename);
		
		return result;
	} else {
		return filename;
	}
}

/*************************************************************************/

} /* namespace misc */

/*************************************************************************/
