/* String-related treasures.
 *
 * PegSoft miscellaneous library (c) 2004 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include <system/headers.h>

#include "misc/string.h"

using namespace std;

/*************************************************************************/

namespace misc
{

/*************************************************************************/

cstring &cstring::operator=(cstring const &right)
{
	if (this != &right)
		copy(right.m_str);
	
	return *this;	
}

/*************************************************************************/

bool cstring::operator==(char const *right) const
{
	if (!m_str && !right)
		return true;
	if (!m_str || !right)
		return false;
	
	return !strcmp(m_str, right);
}

/*************************************************************************/

int cstring::compare(char const *str) const
{ 
	if (!m_str && !str)
		return 0;
	if (!m_str)
		return -1;
	if (!str)
		return 1;
	
	return strcmp(m_str, str); 
}

/*************************************************************************/

int cstring::compare(char const *str, size_t len) const
{
	if (!m_str && !str)
		return 0;
	if (!m_str)
		return -1;
	if (!str)
		return 1;
	
	return strncmp(m_str, str, len); 
}

/*************************************************************************/

void cstring::copy(char const *s)
{
	/* Avoid reallocating if it's not needed. */
	if (s && m_str && (strlen(s) <= strlen(m_str)))
	{
		strcpy(m_str, s);
		return;
	}
	
	free();
	m_str = (s ? nstrdup(s) : 0);
}

/*************************************************************************/

void cstring::copy(char const *s, size_t len)
{
	if (!s)
	{
		copy(s);
		return;
	}

	/* Determine the length to copy. */
	size_t my_len = strlen(s);
	if (my_len > len)
		my_len = len;
	
	reserve(my_len + 1);
	strncpy(m_str, s, my_len);
	m_str[my_len] = '\0';
}

/*************************************************************************/

void cstring::free()
{
	if (m_str)
		delete [] m_str;
}

/*************************************************************************/

char *cstring::reserve(size_t size)
{
	if (size == 0)
		return m_str;
	
	if (m_str && (strlen(m_str) + 1) >= size)
	{
		m_str[0] = '\0';
		return m_str;
	}
	
	free();
	m_str = new char [size];
	
	m_str[0] = '\0';
	return m_str;
}

/*************************************************************************/

void cstring::swap(misc::cstring &str)
{
	char *tempstr = str.m_str;
	str.m_str = m_str;
	m_str = tempstr;
}

/*************************************************************************/

bool insensitive_less::operator()(std::string const &s1, 
		std::string const &s2) const
{
	return ((strcasecmp(s1.c_str(), s2.c_str()) < 0) ? true : false);
}

/*************************************************************************/

char *nstrdup(char const *s)
{
	char *ns = new char[strlen(s) + 1];
	strcpy(ns, s);
	return ns;
}

/*************************************************************************/

bool strtol_range(long *num, char const *str, long min, long max, int base)
{
	errno = 0;

	char *end;
	long result = strtol(str, &end, base);
	
	/* If there is an error, return false immediately. */
	if (errno != 0 || *end != 0)
		return false;
	
	/* Check the range. */
	if (result < min || result > max)
	{
		errno = ERANGE;
		return false;
	}
	
	*num = result;
	return true;
}

/*************************************************************************/

bool strtoul_range(unsigned long *num, char const *str, unsigned long min,
		unsigned long max, int base)
{
	errno = 0;

	char *end;	
	unsigned long result = strtoul(str, &end, base);
	
	/* If there is an error, return false immediately. */
	if (errno != 0 || *end != 0)
		return false;
	
	/* Check the range. */
	if (result < min || result > max)
	{
		errno = ERANGE;
		return false;
	}
	
	*num = result;
	return true;
}

/*************************************************************************/

} /* namespace misc */

/*************************************************************************/
