/* BotServ records management.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */

#include <system/headers.h>

#include "tweak.h"

#include <map>
#include <string>

#include <core/directives.h>
#include <core/epona.h>
#include <databases/database.h>
#include <databases/field.h>

#include "botserv.h"
#include "chanserv.h"
#include "epona.h"
#include "nickserv.h"

#include "services.h"

using namespace std;

using databases::Database;
using databases::Field;
using databases::Record;

/*************************************************************************/

Bots bots;
KBBs kbbs;
BadWords badwords;

databases::Field *BotInfo::f_nick = 0;
databases::Field *BotInfo::f_username = 0;
databases::Field *BotInfo::f_hostname = 0;
databases::Field *BotInfo::f_realname = 0;
databases::Field *BotInfo::f_created = 0;
databases::Field *BotInfo::f_flags = 0;

databases::Field *KBB::f_owner = 0;
databases::Field *KBB::f_name = 0;
databases::Field *KBB::f_count = 0;

databases::Field *BadWord::f_owner = 0;
databases::Field *BadWord::f_word = 0;
databases::Field *BadWord::f_type = 0;

/*************************************************************************/

DefEntry def_kbb[KBB_SIZE] =
{
	DefEntry(KBB_BOLDS,      "BOLDS",      0),
	DefEntry(KBB_COLORS,     "COLORS",     0),
	DefEntry(KBB_REVERSES,   "REVERSES",   0),
	DefEntry(KBB_UNDERLINES, "UNDERLINES", 0),
	DefEntry(KBB_BADWORDS,   "BADWORDS",   0),
	DefEntry(KBB_CAPS,       "CAPS",       0),
	DefEntry(KBB_FLOOD,      "FLOOD",      0),
	DefEntry(KBB_REPEAT,     "REPEAT",     0)
};

/*************************************************************************/

namespace
{

/*************************************************************************/

#define HASH(name)	((static_cast<unsigned char>(tolower((name)[0]))&31)<<5 | (static_cast<unsigned char>(tolower((name)[1]))&31))

class BotHash : public databases::NameCollection::name_hash
{
	public:
		virtual int compare(char const *const &key,
				databases::NameRecord *const &value) const 
				{ return stricmp(key, value->name()); }	
		virtual uint32_t hash(char const *const &key, uint32_t max) const
				{ return HASH(key); }
		virtual bool match(char const *const &key,
				databases::NameRecord *const &value) const
				{ return !stricmp(key, value->name()); }
};

#undef HASH

/*************************************************************************/

BotHash bot_hash;

/*************************************************************************/

typedef std::map<std::string, DefEntry> kbb_map;
kbb_map kbb_byname;

/*************************************************************************/

} /* anonymous namespace */

/*************************************************************************/

BadWord::BadWord() : databases::NameRecord(),
		m_owner(0), m_word(), m_type(bwAny)
{
}

/*************************************************************************/

BadWord::BadWord(identifier_type id) : databases::NameRecord(id),
		m_owner(0), m_word(), m_type(bwAny)
{
}

/*************************************************************************/

databases::Record::identifier_type BadWord::get_ri(Field const *field) const
{
	if (field == f_owner)
		return (m_owner ? m_owner->id() : 0);
	else
		return NameRecord::get_ri(field);
}

/*************************************************************************/

char const *BadWord::get_s(Field const *field) const
{
	if (field == f_word)
		return word();
	else
		return NameRecord::get_s(field);
}

/*************************************************************************/

uint32_t BadWord::get_ui(Field const *field) const
{
	if (field == f_type)
		return type();
	else
		return NameRecord::get_ui(field);
}

/*************************************************************************/

size_t BadWord::memory_usage() const
{
	size_t result = NameRecord::memory_usage();
	
	result += sizeof(BadWord);
	
	if (m_word)
		result += strlen(m_word) + 1;
	
	return result;
}

/*************************************************************************/

ChannelInfo *BadWord::owner(ChannelInfo *owner)
{
	if (m_owner)
		return m_owner;
	
	return (m_owner = owner);
}

/*************************************************************************/

databases::Record::identifier_type BadWord::owner_id(identifier_type owner_id)
{
	/* Assume (db_status() & dbsUpdating == true). */
	
	/* Don't allow owner changes. */
	if (m_owner)
		return m_owner->id();
	
	ChannelInfo *owner;
	
	/* Don't allow no ownership. */
	if (!owner_id || !(owner = (ChannelInfo *)channels.ifind(owner_id)))
	{
		epona->log(ErrorBotServList(), "Ignoring bad word entry #" RECORD_FORMAT
				" with bad owner #" RECORD_FORMAT, id(), owner_id);
		
		/* Update error. */
		db_status_add(dbsUpdateError);
		/* Don't need to change the value, the record will
		 * be deleted. */
		return owner_id;
	}
	
	m_owner = owner;
	return m_owner->id();
}

/*************************************************************************/

Record::identifier_type BadWord::set_ri(Field const *field,
		identifier_type value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ri(field, value);
	
	if (field == f_owner)
		return owner_id(value);
	else
		return NameRecord::set_ri(field, value);
}

/*************************************************************************/

char const *BadWord::set_s(Field const *field, char const *value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_s(field, value);
	
	if (field == f_word)
		return word(value);
	else
		return NameRecord::set_s(field, value);
}

/*************************************************************************/

uint32_t BadWord::set_ui(Field const *field, uint32_t value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ui(field, value);
	
	if (field == f_type)
		return type(value);
	else
		return NameRecord::set_ui(field, value);
}

/*************************************************************************/

uint8_t BadWord::type(uint8_t type)
{
	if (db_status() & dbsUpdating)
	{
		/* If the type is invalid... */
		if (type > bwMax)
		{
			/* If we're already a member, return the old type. */
			if (db_status() & dbsMember)
				return m_type;
			/* Else, make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* Return the bad type, since the record is going
			 * to be deleted anyways. */
			return type;
		}
	}
	
	if (m_type != type)
	{
		if (!badwords.change_ui(this, f_type, type))
			return m_type;
		
		m_type = type;
	}
	
	return m_type;
}

/*************************************************************************/

char const *BadWord::word(char const *word)
{
	if (db_status() & dbsUpdating)
	{
		/* Do not accept NULL words */
		if (!word)
		{
			/* If we're already a member, just return the current word */
			if (db_status() & dbsMember)
				return m_word;
			/* Else, make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* And return the NULL word */
			return word;
		}
		
		/* Do not accept word changes (but allow case changes)  */
		if (m_word && stricmp(m_word, word) != 0)
			return m_word;
	}
	
	if (m_word.compare(word, word_len) != 0)
	{
		misc::cstring tword(word, word_len);
			
		if (!badwords.change_s(this, f_word, tword))
			return m_word;
		
		m_word.swap(tword);
	}
	
	return m_word;
}

/*************************************************************************/

bool BadWords::add(Record *record, int *err)
{
	BadWord *bw = (BadWord *)record;
	
	ChannelInfo::badwords_type::const_iterator it, end;
	
	for (it = bw->owner()->badwords().begin(),
			end = bw->owner()->badwords().end(); it != end; ++it)
	{
		BadWord const *bw2 = (BadWord const *)(*it);
		
		if (!bw2)
			continue;
		
		if (*bw == bw2->name())
		{
			if (err)
				*err = ceName;
			return false;
		}
	}
	
	if (!Collection::add(record, err))
		return false;
	
	return true;
}

/*************************************************************************/

bool BadWords::add_list(Record *record)
{
	BadWord *bw = (BadWord *)record;
	
	ChannelInfo::badwords_type::iterator it, end;
	
	for (it = bw->owner()->badwords().begin(),
			end = bw->owner()->badwords().end(); it != end; ++it)
	{
		if (*it == 0)
		{
			*it = bw;
			break;
		}
	}

	if (it == bw->owner()->badwords().end())
		it = bw->owner()->badwords().insert(it, bw);
	
	if (!Collection::add_list(record))
	{
		*it = 0;
		return false;
	}

	return true;
}

/*************************************************************************/

Record *BadWords::create(Record::identifier_type id) const
{
	if (id)
		return new BadWord(id);
	else
		return new BadWord;
}

/*************************************************************************/

void BadWords::remove_list(Record *record)
{
	BadWord *bw = (BadWord *)record;
	
	ChannelInfo::badwords_type::iterator it, end;
	
	for (it = bw->owner()->badwords().begin(),
			end = bw->owner()->badwords().end(); it != end; ++it)
	{
		if (bw == (*it))
		{
			*it = 0;
			break;
		}
	}
	
	Collection::remove_list(record);
}

/*************************************************************************/

bool BadWords::remove_owned_db(databases::Collection const *coll,
				databases::Record const *record, int *err)
{
	if (coll != &channels)
		return true;
	
	ChannelInfo *ci = (ChannelInfo *)record;
	
	if (ci->badwords().empty())
		return true;
	
	return Collection::remove_owned_db(coll, record, err);
}

/*************************************************************************/

bool BadWords::remove_owned_list(databases::Collection const *coll,
				databases::Record const *record, int *err)
{
	if (coll != &channels)
		return true;
	
	ChannelInfo *ci = (ChannelInfo *)record;
	ChannelInfo::badwords_type::iterator it, end;
	
	for (it = ci->badwords().begin(), end = ci->badwords().end();
			it != end; ++it)
	{
		BadWord *bw = (BadWord *)(*it);
		
		if (bw)
		{
			bw->db_status_remove(Record::dbsInsert);
		
			if (!remove(bw, err))
				return false;
		}
	}
	
	return true;
}

/*************************************************************************/

void BadWords::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("botserv_badwords");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpImport | Database::dpExport);
	
	BadWord::f_owner = db->field_create("owner", 1,
			Field::ftIdentifier, 0, Field::faOwner, "0", 1);
	BadWord::f_word = db->field_create("word", 1, Field::ftString,
			BadWord::word_len, 0, 0, 2);
	BadWord::f_type = db->field_create("type", 1, Field::ftUnsigned,
			1, 0, "0", 3);
	
	if (!db->field_add(BadWord::f_owner) ||
			!db->field_add(BadWord::f_word) ||
			!db->field_add(BadWord::f_type))
		epona->fatal("Can't add fields to database botserv_badwords");
	
	this->db(db);
}

/*************************************************************************/

void BadWords::setup_owner()
{
	channels.set_owned(*this);
}

/*************************************************************************/

BotInfo::BotInfo() : 
		databases::NameRecord(), m_nick(0), m_username(0),
		m_hostname(0), m_realname(0), m_created(time(0)), m_flags(0),
		m_chancount(0), m_lastmsg(time(0))
{
}

/*************************************************************************/

BotInfo::BotInfo(identifier_type id) : 
		databases::NameRecord(id), m_nick(0), m_username(0), m_hostname(0),
		m_realname(0), m_created(time(0)), m_flags(0), m_chancount(0),
		m_lastmsg(time(0))
{
}

/*************************************************************************/

time_t BotInfo::created(time_t created)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't accept dates in the future */
		if (created > time(0))
		{
			if (db_status() & dbsMember)
				return m_created;
			
			created = time(0);
		}
	}
	
	if (m_created != created)
	{
		if (!bots.change_ut(this, f_created, created))
			return m_created;
		
		m_created = created;
	}
	
	return m_created;
}

/*************************************************************************/

bool BotInfo::end_update()
{
	if (!NameRecord::end_update())
		return false;
	
	if (db_status() & dbsUHRUpdated)
		update_client();
	
	return true;
}

/*************************************************************************/

uint16_t BotInfo::flags(uint16_t flags)
{
	/* Assume (db_status() & dbsUpdating) == true */
	
	flags &= ~bfUnused;
	
	if (m_flags != flags)
		m_flags = flags;
	
	return m_flags;
}

/*************************************************************************/

uint16_t BotInfo::flags_add(uint16_t flags)
{
	if ((m_flags & flags) != flags) 
	{
		uint16_t tflags = (m_flags | flags);
		
		if (!bots.change_ui(this, f_flags, tflags))
			return m_flags;
		
		m_flags = tflags;
	}
	
	return m_flags;
}

/*************************************************************************/

uint16_t BotInfo::flags_remove(uint16_t flags)
{
	if ((m_flags & flags) != 0) 
	{
		uint16_t tflags = (m_flags & ~flags);
		
		if (!bots.change_ui(this, f_flags, tflags))
			return m_flags;
		
		m_flags = tflags;
	}
	
	return m_flags;
}
    
/*************************************************************************/

char const *BotInfo::get_s(Field const *field) const
{
	if (field == f_nick)
		return nick();
	else if (field == f_username)
		return username();
	else if (field == f_hostname)
		return hostname();
	else if (field == f_realname)
		return realname();
	else
		return NameRecord::get_s(field);
}

/*************************************************************************/

uint32_t BotInfo::get_ui(Field const *field) const
{
	if (field == f_flags)
		return flags();
	else
		return NameRecord::get_ui(field);
}

/*************************************************************************/

time_t BotInfo::get_ut(Field const *field) const
{
	if (field == f_created)
		return created();
	else
		return NameRecord::get_ut(field);
}

/*************************************************************************/

char const *BotInfo::hostname(char const *hostname)
{
	if (db_status() & dbsUpdating)
	{
		/* Do not accept NULL hostnames */
		if (!hostname)
		{
			/* If we're already a member, just return the current hostname */
			if (db_status() & dbsMember)
				return m_hostname;
			/* Else, make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* And return the NULL hostname */
			return 0;
		}
	}
	
	if (m_hostname.compare(hostname, hostname_len) != 0) 
	{
		misc::cstring thostname(hostname, hostname_len);
		
		if (!bots.change_s(this, f_hostname, thostname))
			return m_hostname;
		
		m_hostname.swap(thostname);
		
		if (db_status() & dbsMember)
			db_status_add(dbsUHRUpdated);
	}
	
	return m_hostname;
}

/*************************************************************************/

size_t BotInfo::memory_usage() const
{
	size_t result = NameRecord::memory_usage();
	
	result += sizeof(BotInfo);
	if (m_nick)
		result += strlen(m_nick) + 1;
	if (m_username)
		result += strlen(m_username) + 1;
	if (m_hostname)
		result += strlen(m_hostname) + 1;
	if (m_realname)
		result += strlen(m_realname) + 1;
	
	return result;
}

/*************************************************************************/

char const *BotInfo::nick(char const *nick)
{
	if (db_status() & dbsUpdating)
	{
		/* Do not accept NULL nicks */
		if (!nick)
		{
			/* If we're already a member, just return the current nick */
			if (db_status() & dbsMember)
				return m_nick;
			/* Else, make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* And return the NULL nick */
			return 0;
		}
		
		/* Some sanity checks for nick changes... */
		if (db_status() & dbsMember)
		{
			if (bots.find(nick) || !bots.nick_cleanup(nick))
				return m_nick;
		}
	}
	
	if (!m_nick)
	{
		m_nick.assign(nick, nick_len);
	} else if (m_nick.compare(nick, nick_len) != 0) 
	{
		misc::cstring tnick(nick, nick_len);
		
		if (!bots.change_s(this, f_nick, tnick))
			return m_nick;
		
		/* Update the record in the collection */
		if (db_status() & dbsMember)
		{
			if (!bots.change_name(this, tnick))
				return m_nick;
		}
		
		s_nick_change(m_nick, tnick, true);
		
		m_nick.swap(tnick);
	}
	
	return m_nick;
}

/*************************************************************************/

char const *BotInfo::realname(char const *realname)
{
	if (db_status() & dbsUpdating)
	{
		/* Do not accept NULL realnames */
		if (!realname)
		{
			/* If we're already a member, just return the current realname */
			if (db_status() & dbsMember)
				return m_realname;
			/* Else, make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* And return the NULL realname */
			return 0;
		}
	}
	
	if (m_realname.compare(realname, realname_len) != 0) 
	{
		misc::cstring trealname(realname, realname_len);
		
		if (!bots.change_s(this, f_realname, trealname))
			return m_realname;
		
		m_realname.swap(trealname);
		
		if (db_status() & dbsMember)
			db_status_add(dbsUHRUpdated);
	}
	
	return m_realname;
}

/*************************************************************************/

char const *BotInfo::set_s(Field const *field, char const *value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_s(field, value);
	
	if (field == f_nick)
		return nick(value);
	else if (field == f_username)
		return username(value);
	else if (field == f_hostname)
		return hostname(value);
	else if (field == f_realname)
		return realname(value);
	else
		return NameRecord::set_s(field, value);
}

/*************************************************************************/

uint32_t BotInfo::set_ui(Field const *field, uint32_t value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ui(field, value);
	
	if (field == f_flags)
		return flags(value);
	else
		return NameRecord::set_ui(field, value);
}

/*************************************************************************/

time_t BotInfo::set_ut(Field const *field, time_t value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ut(field, value);
	
	if (field == f_created)
		return created(value);
	else
		return NameRecord::set_ut(field, value);
}

/*************************************************************************/

void BotInfo::update_client()
{
	if (!(db_status() & dbsMember))
	{
		db_status_remove(dbsUHRUpdated);
		return;
	}
	
	if (db_status() & dbsUHRUpdated)
	{
		/* Set the qlined parameter to false so we don't remove/set
		 * the Q:line again as the nick has already been changed. */
		
		s_quit(nick(), "Be right back", false);
		s_nick(nick(), username(), hostname(), realname(), BOT_MODES, false);
		bot_rejoin_all(this);
		
		db_status_remove(dbsUHRUpdated);
	}
}

/*************************************************************************/

char const *BotInfo::username(char const *username)
{
	if (db_status() & dbsUpdating)
	{
		/* Do not accept NULL usernames */
		if (!username)
		{
			/* If we're already a member, just return the current username */
			if (db_status() & dbsMember)
				return m_username;
			/* Else, make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* And return the NULL username */
			return 0;
		}
	}
	
	if (m_username.compare(username, username_len) != 0) 
	{
		misc::cstring tusername(username, username_len);
		
		if (!bots.change_s(this, f_username, tusername))
			return m_username;
		
		m_username.swap(tusername);
		
		if (db_status() & dbsMember)
			db_status_add(dbsUHRUpdated);
	}
	
	return m_username;
}

/*************************************************************************/

Bots::Bots() : databases::NameCollection(bot_hash)
{
	
}

/*************************************************************************/

bool Bots::add(databases::Record *record, int *err)
{
	if (!NameCollection::add(record, err))
		return false;
	
	BotInfo *bi = (BotInfo *)record;
	
	if (status() & (csInitial | csImport))
	{
		/* Do some cleanup of other databases... */
		if (!nick_cleanup(bi->nick()))
			return false;
	}
	
	/* Introduce the bot on the network */
	if (!(status() & csInitial))
		s_nick(bi->nick(), bi->username(), bi->hostname(), bi->realname(), BOT_MODES, true);
	
	return true;
}

/*************************************************************************/

databases::Record *Bots::create(databases::Record::identifier_type id) const
{
	if (id)
		return new BotInfo(id);
	else
		return new BotInfo;
}

/*************************************************************************/

bool Bots::nick_cleanup(char const *nick) const
{
	User *u;
	NickAlias *na;
	
	/* Check whether the nick is registered, and drop it if so */
	if ((na = aliases.find(nick)))
	{
		if (!aliases.writable() || !groups.writable())
			return false;
		
		if (!aliases.remove(na))
			return false;
	}

	/* Check if a user with this nick is online, and kill him if so */
	if ((u = finduser(nick)))
		kill_user(s_BotServ, u->nick,
				"This nickname is now reserved to services");
	
	return true;
}

/*************************************************************************/

bool Bots::remove(databases::Record *record, int *err)
{
	BotInfo *bi = (BotInfo *)record;

	misc::cstring nick(bi->nick());

	if (!NameCollection::remove(record, err))
		return false;
	
	/* The bot can now leave the network. */
	s_quit(nick, "Help! I'm being deleted!", true);
	
	return true;
}

/*************************************************************************/

void Bots::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("botserv_bots");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpImport | Database::dpExport);
	
	BotInfo::f_nick = db->field_create("nick", 1, Field::ftString,
			BotInfo::nick_len, 0, 0, 1);
	BotInfo::f_username = db->field_create("username", 1, Field::ftString,
			BotInfo::username_len, 0, 0, 2);
	BotInfo::f_hostname = db->field_create("hostname", 1, Field::ftString,
			BotInfo::hostname_len, 0, 0, 3);
	BotInfo::f_realname = db->field_create("realname", 1, Field::ftString,
			BotInfo::realname_len, 0, 0, 4);
	BotInfo::f_created = db->field_create("created", 1, Field::ftUnixTime,
			0, 0, "0", 5);
	BotInfo::f_flags = db->field_create("flags", 1, Field::ftUnsigned,
			2, 0, "0", 6);
	
	if (!db->field_add(BotInfo::f_nick) || 
			!db->field_add(BotInfo::f_username) ||
			!db->field_add(BotInfo::f_hostname) || 
			!db->field_add(BotInfo::f_realname) || 
			!db->field_add(BotInfo::f_created) || 
			!db->field_add(BotInfo::f_flags))
		epona->fatal("Can't add fields to database botserv_bots");
	
	this->db(db);
}

/*************************************************************************/

KBB::KBB() : databases::Record(),
		m_owner(0), m_index(0), m_count(0)
{
}

/*************************************************************************/

KBB::KBB(identifier_type id) : databases::Record(id),
		m_owner(0), m_index(0), m_count(0)
{
}

/*************************************************************************/

uint8_t KBB::count(uint8_t count)
{
	if (m_count != count)
	{
		if (!kbbs.change_ui(this, f_count, count))
			return m_count;
		
		m_count = count;
	}
	
	return m_count;
}

/*************************************************************************/

bool KBB::end_update()
{
	if (!Record::end_update())
		return false;
	
	if (def_kbb[m_index].defval() == count())
	{
		epona->log(InfoChanServList(), "Ignoring kbb "
				RECORD_FORMAT " of channel %s where count == defval ",
				id(), (m_owner ? m_owner->name() : "(unknown)"));
		return false;
	}
	
	return true;
}

/*************************************************************************/

databases::Record::identifier_type KBB::get_ri(Field const *field) const
{
	if (field == f_owner)
		return (m_owner ? m_owner->id() : 0);
	else
		return Record::get_ri(field);
}

/*************************************************************************/

char const *KBB::get_s(Field const *field) const
{
	if (field == f_name)
		return def_kbb[m_index].name();
	else
		return Record::get_s(field);
}

/*************************************************************************/

uint32_t KBB::get_ui(Field const *field) const
{
	if (field == f_count)
		return count();
	else
		return Record::get_ui(field);
}

/*************************************************************************/

uint8_t KBB::index(uint8_t index)
{
	if (db_status() & dbsMember)
		return m_index;
	
	return (m_index = index);
}

/*************************************************************************/

size_t KBB::memory_usage() const
{
	size_t result = Record::memory_usage();
	
	result += sizeof(KBB);
	
	return result;
}

/*************************************************************************/

char const *KBB::name(char const *name)
{
	/* Assume db_status() & dbsUpdating */
	
	/* Do not allow changes */
	if (db_status() & dbsMember)
		return def_kbb[m_index].name();
	
	kbb_map::const_iterator it = kbb_byname.find(name);
	
	/* If the name is not found, make end_update() error */
	if (it == kbb_byname.end())
	{
		db_status_add(dbsUpdateError);
		return name;
	}
	
	m_index = it->second.index();
	return it->second.name();	
}

/*************************************************************************/

ChannelInfo *KBB::owner(ChannelInfo *owner)
{
	if (m_owner)
		return m_owner;
	
	return (m_owner = owner);
}

/*************************************************************************/

databases::Record::identifier_type KBB::owner_id(identifier_type owner_id)
{
	/* Assume (db_status() & dbsUpdating == true). */
	
	/* Don't allow owner changes. */
	if (m_owner)
		return m_owner->id();
	
	ChannelInfo *owner;
	
	/* Don't allow no ownership. */
	if (!owner_id || !(owner = (ChannelInfo *)channels.ifind(owner_id)))
	{
		epona->log(ErrorBotServList(), "Ignoring kbb entry #" RECORD_FORMAT
				" with bad owner #" RECORD_FORMAT, id(), owner_id);
		
		/* Update error. */
		db_status_add(dbsUpdateError);
		/* Don't need to change the value, the record will
		 * be deleted. */
		return owner_id;
	}
	
	m_owner = owner;
	return m_owner->id();
}

/*************************************************************************/

Record::identifier_type KBB::set_ri(Field const *field,
		identifier_type value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_ri(field, value);
	
	if (field == f_owner)
		return owner_id(value);
	else
		return Record::set_ri(field, value);
}

/*************************************************************************/

char const *KBB::set_s(Field const *field, char const *value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_s(field, value);
	
	if (field == f_name)
		return name(value);
	else
		return Record::set_s(field, value);
}

/*************************************************************************/

uint32_t KBB::set_ui(Field const *field, uint32_t value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_ui(field, value);
	
	if (field == f_count)
		return count(value);
	else
		return Record::set_ui(field, value);
}

/*************************************************************************/

bool KBBs::add(Record *record, int *err)
{
	KBB *kbb = (KBB *)record;
	
	if ((kbb->owner()->kbb().size() > kbb->index()) &&
			kbb->owner()->kbb()[kbb->index()])
	{
		epona->log(ErrorBotServList(), "Ignoring duplicate kbb entry #"
				RECORD_FORMAT " from channel %s", kbb->id(),
				kbb->owner()->name());
		return false;
	}
	
	if (!Collection::add(record, err))
		return false;
	
	return true;
}

/*************************************************************************/

bool KBBs::add_list(Record *record)
{
	if (!Collection::add_list(record))
		return false;
	
	KBB *kbb = (KBB *)record;
	
	if (kbb->owner()->kbb().size() <= kbb->index())
		kbb->owner()->kbb().resize(kbb->index() + 1, 0);
	
	kbb->owner()->kbb()[kbb->index()] = kbb;

	return true;
}

/*************************************************************************/

Record *KBBs::create(Record::identifier_type id) const
{
	if (id)
		return new KBB(id);
	else
		return new KBB;
}

/*************************************************************************/

void KBBs::remove_list(Record *record)
{
	KBB *kbb = (KBB *)record;
	
	kbb->owner()->kbb()[kbb->index()] = 0;
	
	Collection::remove_list(record);
}

/*************************************************************************/

bool KBBs::remove_owned_db(databases::Collection const *coll,
				databases::Record const *record, int *err)
{
	if (coll != &channels)
		return true;
	
	ChannelInfo *ci = (ChannelInfo *)record;
	
	if (ci->kbb().empty())
		return true;
	
	return Collection::remove_owned_db(coll, record, err);
}

/*************************************************************************/

bool KBBs::remove_owned_list(databases::Collection const *coll,
				databases::Record const *record, int *err)
{
	if (coll != &channels)
		return true;
	
	ChannelInfo *ci = (ChannelInfo *)record;
	ChannelInfo::kbb_type::iterator it, end;
	
	for (it = ci->kbb().begin(), end = ci->kbb().end();
			it != end; ++it)
	{
		KBB *kbb = (KBB *)*it;
		
		if (kbb)
		{
			kbb->db_status_remove(Record::dbsInsert);
		
			if (!remove(kbb, err))
				return false;
		}
	}
	
	return true;
}

/*************************************************************************/

void KBBs::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("botserv_kbb");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpImport | Database::dpExport);
	
	KBB::f_owner = db->field_create("owner", 1,
			Field::ftIdentifier, 0, Field::faOwner, "0", 1);
	KBB::f_name = db->field_create("name", 1, Field::ftString,
			KBB::name_len, 0, 0, 2);
	KBB::f_count = db->field_create("count", 1, Field::ftUnsigned,
			1, 0, "0", 3);
	
	if (!db->field_add(KBB::f_owner) ||
			!db->field_add(KBB::f_name) ||
			!db->field_add(KBB::f_count))
		epona->fatal("Can't add fields to database botserv_kbb");
	
	this->db(db);
}

/*************************************************************************/

void KBBs::setup_map()
{
	for (int i = 0; i < KBB_SIZE; ++i)
		kbb_byname.insert(make_pair(def_kbb[i].name(), def_kbb[i]));
}

/*************************************************************************/

void KBBs::setup_owner()
{
	channels.set_owned(*this);
}

/*************************************************************************/
