/* Versioning.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * Based on the original code of Services by Andy Church.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/** \file core/version.h
  * \brief Epona release info header file. */

#ifndef CORE_VERSION_H
#define CORE_VERSION_H

/*************************************************************************/

namespace core
{

/*************************************************************************/

/// Program name.
static char const *const program_name = "Epona";
/// Program name in lowercase.
static char const *const program_name_lc = "epona";
/// Program name in uppercase
static char const *const program_name_uc = "EPONA";
/// Program version.
static unsigned int const program_version = 0x01050200;

/*************************************************************************/

/// Possible values for program status.
enum program_status_t
{
	/// Development snapshot.
	psDevel  = 0x01,
	/// Alpha release (not feature complete)
	psAlpha  = 0x02,
	/// Beta release (feature complete)
	psBeta   = 0x04,
	/// Pre-release (feature complete, works in production, needs more tests)
	psPre    = 0x08,
	/// Stable release (feature complete, no problems reported)
	psStable = 0x10
};

static program_status_t const program_status = psPre; 

/*************************************************************************/

/// Program release date.
static int const program_date = 20070815;

/*************************************************************************/

/// Core module version.
static int const core_version = program_version;

/*************************************************************************/

#if defined(IRC_SEQUANA)
static char const *const program_protocol = "Sequana 1.4.36p2+";
static char const *const program_protocol_l = "S";
#elif defined(IRC_SEQUANA_LIGHT)
static char const *const program_protocol = "Sequana Light 1.8.3+";
static char const *const program_protocol_l = "s";
#elif defined(IRC_BAHAMUT)
static char const *const program_protocol = "Bahamut 1.4.36+";
static char const *const program_protocol_l = "B";
#elif defined(IRC_ULTIMATE)
static char const *const program_protocol = "UltimateIRCd 2.8.2+";
static char const *const program_protocol_l = "u";
#elif defined(IRC_UNREAL)
static char const *const program_protocol = "UnrealIRCd 3.2+";
static char const *const program_protocol_l = "U";
#elif defined(IRC_DREAMFORGE)
static char const *const program_protocol = "DreamForge 4.6.7";
static char const *const program_protocol_l = "D";
#else
static char const *const program_protocol = "unknown";
static char const *const program_protocol_l = "unknown";
#endif

/*************************************************************************/

} /* namespace core */

/*************************************************************************/

#endif /* CORE_VERSION_H */
