/* Database records and collection classes.
 *
 * Configuration data module (c) 2006 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <databases/database.h>
#include <databases/field.h>
#include <modules/core.h>

#include "decl.h"
#include "module.h"
#include "records.h"

using namespace std;

using databases::Database;
using databases::Field;
using databases::Record;

/*************************************************************************/

namespace confdata
{

/*************************************************************************/

Field *ConfDir::f_name = 0;
Field *ConfDir::f_value = 0;

/*************************************************************************/

ConfDir::ConfDir() : Record(), m_name(), m_value()
{
}

/*************************************************************************/

ConfDir::ConfDir(identifier_type id) : Record(id), m_name(), m_value()
{	
}

/*************************************************************************/

char const *ConfDir::get_s(Field const *field) const
{
	if (field == f_name)
		return name();
	else if (field == f_value)
		return value();
	else
		return Record::get_s(field);
}

/*************************************************************************/

size_t ConfDir::memory_usage() const
{
	size_t result = Record::memory_usage();

	result += sizeof(ConfDir);
	result += strlen(m_name) + 1;
	result += strlen(m_value) + 1;

	return result;
}

/*************************************************************************/

char const *ConfDir::name(char const *name)
{
	if (m_name.compare(name, name_len) != 0)
	{
		misc::cstring tname(name, name_len);
		
		if (!me->confdirs().change_s(this, f_name, tname))
			return m_name;
		m_name.swap(tname);
	}
	
	return m_name;
}

/*************************************************************************/

char const *ConfDir::value(char const *value)
{
	if (m_value.compare(value, value_len) != 0)
	{
		misc::cstring tvalue(value, value_len);
		
		if (!me->confdirs().change_s(this, f_value, tvalue))
			return m_value;
		m_value.swap(tvalue);
	}
	
	return m_value;
}

/*************************************************************************/

Record *ConfDirs::create(Record::identifier_type id) const
{
	if (id)
		return new ConfDir(id);
	else
		return new ConfDir;
}

/*************************************************************************/

void ConfDirs::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("confdata_directives");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpExport);
	
	ConfDir::f_name = db->field_create("name", 1, Field::ftString,
			ConfDir::name_len, 0, NULL, 1);
	ConfDir::f_value = db->field_create("value", 1, Field::ftString,
			ConfDir::value_len, 0, NULL, 2);
	
	if (!db->field_add(ConfDir::f_name) || 
			!db->field_add(ConfDir::f_value))
		me->core().fatal("Can't add fields to database confdata_directives");
	
	this->db(db);
}

/*************************************************************************/

} /* namespace confdata */

/*************************************************************************/
