/* Log collection.
 *
 * Log file module (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <log/levels.h>
#include <modules/core.h>
#include <modules/module.h>

#include "coll.h"
#include "file.h"
#include "decl.h"
#include "module.h"

using namespace std;

using modules::Module;
using xlog::LogLevel;

/*************************************************************************/

namespace logfile
{

/*************************************************************************/

Collection::~Collection()
{
	clear();
}

/*************************************************************************/

bool Collection::add(hidden::Directives::File const &directive)
{
	File *file = find(directive.filename.value());
	
	if (file)
	{	
		if (file->marked())
		{
			me->core().log(LogFileError(), "File %s is already configured",
					directive.filename.value().c_str());
			return false;
		}
		
		file->directive(directive);
		if (!file->ready())
		{
			remove(file);
			return false;
		}
		
		file->marked(true);
		return true;
	} 
	
	file = new File(directive);
	if (!file->ready())
	{
		delete file;
		return false;
	}
	
	m_members.push_back(file);
	return true;
}

/*************************************************************************/

void Collection::cleanup()
{
	container_type::iterator it, end, cur;
	
	for (it = m_members.begin(), end = m_members.end(); (cur = it++) != end; )
	{	
		if (!(*cur)->marked())
		{
			delete (*cur);
			m_members.erase(cur);
		} else {
			(*cur)->marked(false);
		}
	}
}

/*************************************************************************/

void Collection::clear()
{
	container_type::iterator it, end;
	
	for (it = m_members.begin(), end = m_members.end(); it != end; ++it)
		delete (*it);
	
	m_members.clear();
}

/*************************************************************************/

File *Collection::find(std::string const &name)
{
	container_type::iterator it, end;
	
	for (it = m_members.begin(), end = m_members.end(); it != end; ++it)
		if ((*it)->directive().filename == name)
			return (*it);
	
	return 0;
}

/*************************************************************************/

void Collection::log(LogLevel const &level, char const *message)
{
	container_type::iterator it, end;
	
	for (it = m_members.begin(), end = m_members.end(); it != end; ++it)
		(*it)->log(level, message);
}

/*************************************************************************/

void Collection::module_notify(modules::Module &sender, int msg)
{
	container_type::iterator it, end;
	
	for (it = m_members.begin(), end = m_members.end(); it != end; ++it)
	{
		if ((*it)->directive().rotate &&
				(*it)->directive().rotate.compress &&
				((*it)->directive().rotate.compress.module == sender.name()))
		{
			if (msg == Module::mnStarted) {	
				(*it)->set_compressor(&sender);
			} else if (msg == Module::mnStopped) {
				(*it)->set_compressor(0);
			}
		}
	}
}

/*************************************************************************/

void Collection::relocate()
{
	container_type::iterator it, end, cur;
	
	for (it = m_members.begin(), end = m_members.end(); (cur = it++) != end; )
	{	
		if ((*cur)->directive().filename[0] != '/')
		{	
			(*cur)->close();
			
			if (!(*cur)->open())
			{
				delete (*cur);
				m_members.erase(cur);
			}
		}
	}	
}

/*************************************************************************/

bool Collection::remove(File *file)
{
	container_type::iterator it, end;
	
	for (it = m_members.begin(), end = m_members.end(); it != end; ++it)
	{
		if ((*it) == file)
		{
			delete (*it);
			m_members.erase(it);
			return true;
		}
	}
	
	return false;
}

/*************************************************************************/

} /* namespace logfile */

/*************************************************************************/
