/* Log file module.
 *
 * Log file module (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MODULE_H
#define MODULE_H

/*************************************************************************/

#include <log/log.h>
#include <modules/module.h>

#include "coll.h"

/*************************************************************************/

namespace modules { class Core; }
namespace xlog { class LogLevel; }

/*************************************************************************/

namespace logfile
{

/*************************************************************************/

namespace hidden { class Directives; }

/*************************************************************************/

static int const logfile_version = 0x01000000;

/*************************************************************************/

class LogFile : public modules::Module
{
	public:
		/// Constructor.
		LogFile(modules::Core &core);
		/// Destructor.
		~LogFile();
		
		/// Overrides Module::config().
		virtual conf::ConfQ *config(modules::Core &core,
				conf::ConfGlobalQ &global);
		
		/// Overrides Module::load().
		virtual bool load(modules::Core &core);
		
		/// Overrides Module::notify().
		virtual void notify(modules::Module &sender, int msg, void *arg);
		
		/// Overrides Module::start().
		virtual void start(modules::Core &core);
		
		/// Overrides Module::stop().
		virtual bool stop(modules::Core &core);
		
		/// Overrides Module::version().
		virtual int version() const { return logfile_version; }
		
		/// Returns the core module.
		modules::Core &core() { return *m_core; }
		
		/// Returns the configuration directives.
		hidden::Directives const &cd() const { return *m_cd; }
		
		/// Returns the log file collection.
		Collection &collection() { return m_collection; }
		
		/// Returns the scheduler task type registered by the module.
		int task_type() const { return m_tasktype; }
	private:
		/* Classes */
		
		class Intf : public xlog::Log
		{
			virtual void log(xlog::LogLevel const &level, char const *message);
		};
	
		/* Data */
		
		modules::Core *m_core;
		Intf m_intf;
		hidden::Directives *m_cd;
		Collection m_collection;
		int m_tasktype;
};

/*************************************************************************/

} /* namespace logfile */

/*************************************************************************/

#endif /* MODULE_H */
