/* MySQL result set class.
 *
 * MySQL client module (c) 2006 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <mysql-cli/result.h>

using namespace std;
using namespace mysql_cli;

/*************************************************************************/

namespace mysql_cli
{

/*************************************************************************/

Result::Result(MYSQL_RES *res) : m_sh(0)
{
	if (res)
		m_sh = new Shared(res);
}

/*************************************************************************/

Result::Result(Result const &r) : m_sh(r.m_sh)
{
	if (m_sh)
		m_sh->refcount()++;
}

/*************************************************************************/

Result &Result::operator=(Result const &right)
{
	if (this != &right)
	{
		if (m_sh)
		{
			m_sh->refcount()--;
			if (m_sh->refcount() == 0)
				delete m_sh;
		}
		
		m_sh = right.m_sh;
		
		if (m_sh)
			m_sh->refcount()++;
	}
	
	return *this;
}

/*************************************************************************/

Result &Result::operator=(MYSQL_RES *right)
{
	if (m_sh)
	{
		m_sh->refcount()--;
		if (m_sh->refcount() == 0)
			delete m_sh;
	}
		
	m_sh = new Shared(right);
	
	return *this;
}

/*************************************************************************/

Result::~Result()
{
	if (m_sh)
	{
		m_sh->refcount()--;
		if (m_sh->refcount() == 0)
			delete m_sh;
	}
}

/*************************************************************************/

void Result::data_seek(my_ulonglong offset)
{
	if (m_sh)
		mysql_data_seek(m_sh->res(), offset);
}

/*************************************************************************/

MYSQL_FIELD *Result::fetch_field()
{
	if (!m_sh)
		return 0;
	
	return mysql_fetch_field(m_sh->res());
}

/*************************************************************************/

MYSQL_FIELD *Result::fetch_field(unsigned int fieldnum)
{
	if (!m_sh)
		return 0;
	
	return mysql_fetch_field_direct(m_sh->res(), fieldnum);
}

/*************************************************************************/

MYSQL_FIELD *Result::fetch_fields()
{
	if (!m_sh)
		return 0;
	
	return mysql_fetch_fields(m_sh->res());
}

/*************************************************************************/

unsigned long *Result::fetch_lengths()
{
	if (!m_sh)
		return 0;
	
	return mysql_fetch_lengths(m_sh->res());
}

/*************************************************************************/

MYSQL_ROW Result::fetch_row()
{
	if (!m_sh)
		return 0;
	
	return mysql_fetch_row(m_sh->res());
}

/*************************************************************************/

MYSQL_FIELD_OFFSET Result::field_seek(MYSQL_FIELD_OFFSET offset)
{
	if (!m_sh)
		return 0;
	
	return mysql_field_seek(m_sh->res(), offset);
}

/*************************************************************************/

MYSQL_FIELD_OFFSET Result::field_tell()
{
	if (!m_sh)
		return 0;
	
	return mysql_field_tell(m_sh->res());
}

/*************************************************************************/

unsigned int Result::num_fields()
{
	if (!m_sh)
		return 0;
	
	return mysql_num_fields(m_sh->res());
}

/*************************************************************************/

my_ulonglong Result::num_rows()
{
	if (!m_sh)
		return 0;
	
	return mysql_num_rows(m_sh->res());
}

/*************************************************************************/

MYSQL_ROW_OFFSET Result::row_seek(MYSQL_ROW_OFFSET offset)
{
	if (!m_sh)
		return 0;
	
	return mysql_row_seek(m_sh->res(), offset);
}

/*************************************************************************/

MYSQL_ROW_OFFSET Result::row_tell()
{
	if (!m_sh)
		return 0;
	
	return mysql_row_tell(m_sh->res());
}

/*************************************************************************/

Result::Shared::~Shared()
{
	mysql_free_result(m_res);
}

/*************************************************************************/

} /* namespace mysql_cli */

/*************************************************************************/
