/* Configuration.
 *
 * MySQL database module (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef CONF_H
#define CONF_H

/*************************************************************************/

#include <conf/directives.h>
#include <conf/quick.h>

/*************************************************************************/
	
namespace mysql 
{

/*************************************************************************/

class Directives : public conf::DirectivesQ
{
	public:
		/* Classes */
		class Connection : public conf::ConfBlock
		{
			public:
				/* Data */
				conf::ConfStrChar hostname;
				conf::ConfPort port;
				conf::ConfStrChar unix_socket;
				conf::ConfStrChar username;
				conf::ConfStrChar password;
				conf::ConfStrChar database;
				conf::ConfDuration timeout;
				conf::ConfStrChar charset;
				conf::ConfBool log_queries;
				conf::ConfBool log_warnings;
				
				/* Functions */
				Connection();
			protected:
				virtual bool add(conf::Conf &conf, conf::Conf &new_conf);
				virtual bool apply_changes(conf::Conf &conf);
		};
		
		class ReconnectFrequency : public conf::ConfDuration
		{
			public:
				/* Functions */
				ReconnectFrequency();
			protected:
				virtual bool apply_change(conf::Conf &conf);
		};
		
		class Sync : public conf::ConfBlock
		{
			public:
				/* Classes */
				class Frequency : public conf::ConfDuration
				{
					public:
						/* Functions */
						Frequency();
					protected:
						virtual bool apply_change(conf::Conf &conf);
				}; 
			
				/* Data */
				Frequency frequency;
				conf::ConfStrCharA blacklist;
				conf::ConfBool mem_usage;
				
				/* Functions */
				Sync();
			protected:
				virtual bool add(conf::Conf &conf, conf::Conf &new_conf);
		};
		
		/* Data */
		Connection connection;
		ReconnectFrequency reconnect_frequency;
		Sync sync;
		conf::ConfUint<uint32_t> select_max;
		conf::ConfStrChar table_prefix;
		conf::ConfStrChar table_charset;
		conf::ConfStrChar table_collation;
		
		/* Functions */ 
		
		/// Constructor.
		Directives(conf::ConfLogger &logger);
		
		/// Overrides DirectivesQ::add().
		bool add(conf::Conf &conf);
	private:
		Directives(Directives const &d);
		Directives &operator=(Directives const &right);
};

/*************************************************************************/

} /* namespace mysql */

/*************************************************************************/

#endif /* CONF_H */
